%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This is a demo code for the algorithm proposed in 
%   P. Qi, X. Hu, "Learning nonlinear statistical regularities in natural images by 
%   modeling the outer product of image intensities," Neural Computation, 
%   vol. 26, no. 4, pp. 693C711, 2014
%
% To run the demo, 
%   1. Download outer_prod.zip and unzip it
%   2. Change the current path of Matlab to the folder outer_prod
%   3. Make a 3-D array of image dataset X with dimensions 
%      Height x Width x NumberOfImages. Save it to the current folder.
%   4. Run demo_outer_product.m
%
% If you find any bug in the codes, please report it to: 
%   pengrobertqi@163.com   
% Thank you.
%
% Written by Peng Qi @ Stanford University
% Last updated Jan. 11, 2014
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Load dataset
fprintf('Loading dataset...\n');
load X; % X should be a 3-D array of the dataset with dimensions 
        % Height x Width x NumberOfImages

patchsize = 12;
retainedDims = 100;
N = 100000;
numbases = 100; % Train 100 first-layer units
numbases2 = 10; % Train 10 second-layer units
gamma = 0.3;    % L1 regularization

% Compute whitening matrices
fprintf('Computing whitening matrices...\n');
compWhitMatrix(patchsize, retainedDims);

% Load computed whitening matrices
load(sprintf('whitM_%d_%d', patchsize^2, retainedDims));

% Get patches from the dataset and whiten them
fprintf('Sampling patches...\n');
x = getBatch(X, patchsize, N);
x = bsxfun(@minus, x, mean(x));
x = whM * x;

% Compute first-layer parameters
fprintf('Computing first-layer parameters...\n');
[B, S] = reg_outer_prod(x, eye(numbases), numbases, numbases, gamma, 20,...
    1000, .1, @(B) showcov(B, uwhM));

% Compute second-layer parameters
fprintf('Computing second-layer parameters...\n');
[W, S] = reg_outer_prod2(x, B, numbases, numbases2, 10*gamma, 10, 1000, .03, ...
    @(B) showcov(B, uwhM));